import { parsePath, createPath, createMemoryHistory, createBrowserHistory, createHashHistory } from 'history';
import { useContext, createContext, useMemo, useState, useEffect } from 'react';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';
import { match as match$1 } from 'path-to-regexp';

const ViewContext = /* @__PURE__ */ createContext(null);
function ViewProvider(props) {
  return /* @__PURE__ */ jsx(ViewContext.Provider, {
    ...props
  });
}
function useView() {
  return useContext(ViewContext);
}
function View() {
  return useView();
}
const DataContext = /* @__PURE__ */ createContext([void 0, {}]);
function useDataContext() {
  return useContext(DataContext);
}
function useNamedData() {
  return useDataContext()[1];
}
function DataProvider(_ref) {
  let {
    children,
    name,
    data
  } = _ref;
  const namedData = useNamedData();
  const value = useMemo(() => [data, name ? {
    ...namedData,
    [name]: data
  } : namedData], [data, name, namedData]);
  return /* @__PURE__ */ jsx(DataContext.Provider, {
    value,
    children
  });
}
const MatchedContext = /* @__PURE__ */ createContext(void 0);
function useMatched() {
  return useContext(MatchedContext);
}
function useData(name) {
  const [data, namedData] = useDataContext();
  return name ? namedData[name] : data;
}
const LoadingContext = /* @__PURE__ */ createContext(void 0);
function useLoading() {
  return useContext(LoadingContext);
}

let i = 1;
function uniqId() {
  return i++;
}
function noop() {
}
const reject = /* @__PURE__ */ Promise.reject.bind(Promise);
function cancelPromise() {
  return new Promise(noop);
}
function createCurrentGuard() {
  let current;
  return [function currentGuard(promise) {
    const cur = uniqId();
    current = cur;
    return promise.then((result) => current === cur ? result : cancelPromise()).catch((err) => current === cur ? Promise.reject(err) : cancelPromise());
  }, function cancel() {
    current = void 0;
  }];
}
function splitProps(obj, keys) {
  const picked = {};
  const rest = {
    ...obj
  };
  keys.forEach((key) => {
    picked[key] = rest[key];
    delete rest[key];
  });
  return [picked, rest];
}
function isString(maybeString) {
  return typeof maybeString === "string";
}

class NativeRouterError extends Error {
}
class NotFoundError extends NativeRouterError {
  constructor(pathname) {
    super(`Can't find the path: ${pathname}`);
  }
}

function create(routes, history, resolveView, options) {
  const [currentGuard, cancelAll] = createCurrentGuard();
  const {
    index,
    locationStack
  } = getHistoryState({
    history
  });
  const viewStack = [];
  if (options?.currentView) {
    viewStack[index] = options.currentView;
  }
  return {
    routes: Array.isArray(routes) ? routes : [routes],
    resolveView,
    history,
    locationStack,
    viewStack,
    currentGuard,
    cancelAll,
    errorHandler: reject,
    ...options,
    baseUrl: options?.baseUrl || ""
  };
}
function setOptions(router, options) {
  return Object.assign(router, options);
}
function getHistoryState(_ref) {
  let {
    history
  } = _ref;
  const state = history.location.state || {};
  return {
    index: state.index || 0,
    locationStack: state.locationStack || [history.location]
  };
}
function getCurrentView(router) {
  return router.viewStack[getHistoryState(router).index];
}
function match(router, pathname) {
  function matchRoutes(routes, baseUrl, pathname2) {
    for (let i = 0; i < routes.length; i++) {
      const route = routes[i];
      const end = !route.children;
      const matched = route.path ? match$1(route.path, {
        strict: true,
        sensitive: true,
        decode: typeof decodeURIComponent === "function" ? decodeURIComponent : void 0,
        end
      })(pathname2) : {
        path: "",
        index: 0,
        params: {}
      };
      if (matched) {
        const result = {
          route,
          ...matched
        };
        if (end)
          return [result];
        const children = matchRoutes(route.children, `${baseUrl}${route.path || ""}`, pathname2.slice(matched.path.length));
        if (children)
          return [result, ...children];
        return void 0;
      }
    }
    return void 0;
  }
  return matchRoutes(router.routes, router.baseUrl, pathname.slice(router.baseUrl.length));
}
function toLocation(router, to, state) {
  const {
    baseUrl
  } = router;
  return {
    pathname: "",
    search: "",
    hash: "",
    ...parsePath(baseUrl + to),
    state
  };
}
function resolve(router, location) {
  const matched = match(router, location.pathname);
  const {
    resolveView,
    errorHandler
  } = router;
  return (matched ? resolveView(matched, {
    router,
    location
  }) : Promise.reject(new NotFoundError(location.pathname))).catch(errorHandler);
}
function resolveTo(router, to, state) {
  const location = toLocation(router, to, state);
  return resolve(router, location);
}
function commit(router, resolvePromise, location) {
  const {
    history,
    currentGuard,
    onLoadingChange = noop
  } = router;
  const nextIndex = getHistoryState(router).index + 1;
  if (router.resolving) {
    onLoadingChange();
  }
  router.resolving = location;
  onLoadingChange("pending");
  return currentGuard(resolvePromise).then((resolvedView) => {
    router.locationStack = [...router.locationStack.slice(0, nextIndex), location];
    router.viewStack = [...router.viewStack.slice(0, nextIndex), resolvedView];
    history.push(location, {
      index: nextIndex,
      locationStack: router.locationStack
    });
    onLoadingChange("resolved");
  }).catch((e) => {
    onLoadingChange("rejected");
    throw e;
  });
}
function commitReplace(router, resolvePromise, location) {
  const {
    history,
    currentGuard,
    onLoadingChange = noop
  } = router;
  const {
    index
  } = getHistoryState(router);
  if (router.resolving) {
    onLoadingChange();
  }
  router.resolving = location;
  onLoadingChange("pending");
  return currentGuard(resolvePromise).then((resolvedView) => {
    router.locationStack[index] = location;
    router.viewStack[index] = resolvedView;
    history.replace(location, {
      index,
      locationStack: router.locationStack
    });
    onLoadingChange("resolved");
  }).catch((e) => {
    onLoadingChange("rejected");
    throw e;
  });
}
function navigate(router, to, state) {
  const location = toLocation(router, to, state);
  const viewPromise = resolve(router, location);
  return commit(router, viewPromise, location);
}
function refresh(router) {
  const {
    location
  } = router.history;
  const viewPromise = resolve(router, location);
  return commitReplace(router, viewPromise, location);
}
function go(router, delta) {
  router.history.go(delta);
}
function forward(router) {
  router.history.forward();
}
function back(router) {
  router.history.back();
}
function createHref(_ref2, to) {
  let {
    baseUrl,
    history
  } = _ref2;
  return baseUrl + history.createHref(to);
}
function cancel(_ref3) {
  let {
    cancelAll,
    onLoadingChange = noop
  } = _ref3;
  cancelAll();
  onLoadingChange();
}
function initHistoryStack(router) {
  const {
    history
  } = router;
  const {
    locationStack
  } = getHistoryState(router);
  return Promise.all(locationStack.map((l) => resolve(router, l))).then((views) => {
    router.viewStack = views;
    history.replace(createPath(history.location), history.location.state);
  });
}
function listen(router, onViewChange) {
  const {
    history
  } = router;
  const rmListener = history.listen((_ref4) => {
    let {
      action,
      location
    } = _ref4;
    cancel(router);
    const state = location.state;
    const index = state?.index || 0;
    const view = router.viewStack[index];
    onViewChange(view);
    if (!view)
      refresh(router);
    if (action === "POP") {
      history.replace(createPath(history.location), {
        ...state,
        locationStack: router.locationStack
      });
    }
  });
  history.replace(createPath(history.location), history.location.state);
  return () => {
    cancel(router);
    rmListener();
  };
}

function resolveView(matched, ctx) {
  return resolveViewBase(matched, ctx, (data, dataCtx) => data?.(dataCtx));
}
const viewDataMap = /* @__PURE__ */ new WeakMap();
function resolveViewServer(matched, ctx) {
  const dataResults = new Array(matched.length);
  return resolveViewBase(matched, ctx, (data, dataCtx) => Promise.resolve(data?.(dataCtx)).then((result) => dataResults[dataCtx.index] = result)).then((view) => {
    viewDataMap.set(view, dataResults);
    return view;
  });
}
function createHydrateResolveView(data) {
  return (matched, ctx) => resolveViewBase(matched, ctx, (_, dataCtx) => data[dataCtx.index]);
}
function getViewData(view) {
  return viewDataMap.get(view);
}
function resolveViewBase(matched, _ref, resolveData) {
  let {
    router,
    location
  } = _ref;
  return Promise.all(matched.map((_ref2, index) => {
    let {
      params,
      route
    } = _ref2;
    const ctx = {
      matched,
      params,
      index,
      router,
      location
    };
    function resolveComponent() {
      if (!route.component)
        return View;
      const r = route.component(ctx);
      return Promise.resolve(r).then((m) => "default" in m ? m.default : m);
    }
    return Promise.all([resolveData(route.data, ctx), resolveComponent()]).then((_ref3) => {
      let [data, C] = _ref3;
      return /* @__PURE__ */ jsx(DataProvider, {
        data,
        name: route.name,
        children: /* @__PURE__ */ jsx(MatchedContext.Provider, {
          value: ctx,
          children: /* @__PURE__ */ jsx(C, {})
        })
      });
    });
  })).then((views) => views.reverse().reduce((acc, view) => /* @__PURE__ */ jsx(ViewProvider, {
    value: acc,
    children: view
  })));
}

const RouterContext = /* @__PURE__ */ createContext(null);
function Router(_ref) {
  let {
    router,
    children
  } = _ref;
  const [view, setView] = useState(getCurrentView(router));
  useEffect(() => listen(router, setView), [router]);
  return /* @__PURE__ */ jsx(RouterContext.Provider, {
    value: router,
    children: children === "undefined" ? view : /* @__PURE__ */ jsx(ViewProvider, {
      value: view,
      children
    })
  });
}
function createRouter(routes, history) {
  let {
    resolveView: resolveView$1 = resolveView,
    ...options
  } = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : {};
  return create(routes, history, resolveView$1, options);
}
function useNewRouter(_ref2, createHistory) {
  let {
    routes,
    children,
    ...options
  } = _ref2;
  const [tracked, rest] = splitProps(options, ["baseUrl", "currentView"]);
  const router = useMemo(() => createRouter(routes, createHistory(), tracked), [routes, createHistory, ...Object.keys(tracked), ...Object.values(tracked)]);
  const [loading, setLoading] = useState();
  setOptions(router, {
    ...rest,
    onLoadingChange(status) {
      setLoading(status && {
        key: uniqId(),
        status
      });
    }
  });
  const r = useMemo(() => /* @__PURE__ */ jsx(Router, {
    router,
    children
  }), [router, children]);
  return /* @__PURE__ */ jsx(LoadingContext.Provider, {
    value: loading,
    children: r
  });
}
function HistoryRouter(props) {
  return useNewRouter(props, createBrowserHistory);
}
function HashRouter(props) {
  return useNewRouter(props, createHashHistory);
}
function MemoryRouter(_ref3) {
  let {
    initialEntries,
    initialIndex,
    ...props
  } = _ref3;
  const createHistory = useMemo(() => () => createMemoryHistory({
    initialEntries,
    initialIndex
  }), [initialEntries, initialIndex]);
  return useNewRouter(props, createHistory);
}
function useRouter() {
  return useContext(RouterContext);
}

const defaultHydrateKey = "_nativeRouterReactSSRData";
function resolveServerViewBase(router, location, options) {
  return resolve(router, location).then((view) => {
    const data = getViewData(view);
    return /* @__PURE__ */ jsxs(Fragment, {
      children: [/* @__PURE__ */ jsx(Router, {
        router,
        children: view
      }), /* @__PURE__ */ jsx("script", {
        ...options?.scriptAttributes,
        suppressHydrationWarning: true,
        dangerouslySetInnerHTML: {
          __html: `window.${options?.hydrateKey || defaultHydrateKey} = ${JSON.stringify({
            data,
            location
          })};`
        }
      })]
    });
  });
}
function resolveServerView(routes, location) {
  let {
    scriptAttributes,
    hydrateKey,
    ...options
  } = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : {};
  const router = create(routes, createMemoryHistory({
    initialEntries: [location]
  }), resolveViewServer, options);
  return resolveServerViewBase(router, isString(location) ? toLocation(router, location) : location, {
    scriptAttributes,
    hydrateKey
  });
}
function resolveClientView(routes, options) {
  const {
    data,
    location
  } = window[options?.hydrateKey || defaultHydrateKey];
  const router = create(routes, createMemoryHistory({
    initialEntries: [location]
  }), createHydrateResolveView(data), options);
  return resolve(router, location);
}

export { initHistoryStack as A, listen as B, NotFoundError as C, resolveServerView as D, HistoryRouter as H, MemoryRouter as M, NativeRouterError as N, Router as R, View as V, commit as a, useView as b, createHref as c, useData as d, useLoading as e, useMatched as f, resolveView as g, resolveClientView as h, createRouter as i, HashRouter as j, create as k, getHistoryState as l, getCurrentView as m, navigate as n, match as o, resolveTo as p, commitReplace as q, resolve as r, setOptions as s, toLocation as t, useRouter as u, refresh as v, go as w, forward as x, back as y, cancel as z };
//# sourceMappingURL=server-51cb91a1.mjs.map
